#!/usr/bin/env python3
# coding: utf-8

import configparser
import uuid
from pathlib import Path

import speedtest


class _MyConfigParser(configparser.ConfigParser):
    def __init__(self, *args, **kwargs) -> None:
        super(_MyConfigParser, self).__init__(*args, **kwargs)

    def write2(self, file_name) -> None:
        with open(file_name, 'w') as cf:
            self.write(cf)


class _Dict(dict):
    def __init__(self, *args, **kwargs) -> None:
        super(_Dict, self).__init__(*args, **kwargs)
        self.__dict__ = self


class _Config:
    def __init__(self, file_name: str = 'config.ini') -> None:
        self.file_name = file_name
        self.config = _MyConfigParser(allow_no_value=True)
        self.project_path = str(Path(__file__).parent)

    def __call__(self) -> _MyConfigParser:
        if not Path(self.file_name).exists():
            print('Creating a config file...')
            config = self.create_config()
            config.write2(self.file_name)
        else:
            config = self.read()
        if (
                config['main']['upload_speed'] == '' or
                config['main']['uuid'] != str(uuid.getnode())  # noqa
        ):
            uspeed = self.upload_speed()
            config['main']['upload_speed'] = str(uspeed)
            config.write2(self.file_name)
        return config

    def read(self):
        conf = self.config
        conf.read(self.file_name)
        return conf

    def create_config(self) -> _MyConfigParser:
        self.config.add_section('main')
        conf = self.config['main']
        conf['project_path'] = self.project_path
        conf['ublock'] = f'{self.project_path}/.addons/ublock_latest.xpi'
        conf['data_path'] = f'{self.project_path}/data'
        conf['log_file'] = 'pymirror.log'
        conf['win_gecko'] = ''
        conf['uuid'] = str(uuid.getnode())
        conf['upload_speed'] = ''
        return self.config

    def get_dict(self) -> _Dict:
        self.config.read(self.file_name)
        config_dict = dict(self.config['main'].items())
        return _Dict(config_dict)

    @staticmethod
    def upload_speed():
        st = speedtest.Speedtest()
        speed = st.upload()
        return speed / 8e+6


_Config().__call__()
_config = _Config().get_dict()
Config = _config



#!/usr/bin/env python3
# coding: utf-8

import argparse
import json
import shlex
import subprocess
import time
from pathlib import Path
from typing import Union, Optional

from dracula import DraculaPalette as Dp

from .config import Config
from .helpers import Shared, console
from .start_driver import StartDrive


class MultiUp:
    def __init__(self, args: argparse.Namespace):
        self.args = args

    def _multiup(self, driver) -> Optional[list]:
        def cURL_request(url: str) -> Union[None, dict]:
            cURL = shlex.split(url)
            try:
                out = subprocess.run(cURL, stdout=subprocess.PIPE, check=True)
            except subprocess.CalledProcessError:
                console.print(f'{Dp.r}Encountered an unexpected issue with '
                              'multiup')
                return
            res = out.stdout.decode('UTF-8').replace('\\', '')
            try:
                res = json.loads(res)
            except json.JSONDecodeError as error:
                raise error
            return res

        multiup_links = []

        with open(f'{Config.data_path}/more_links.json') as j:
            more_links = json.load(j)

        server = cURL_request(
            'curl -s https://www.multiup.org/api/get-fastest-server'
        )['server']
        selected_hosts_lst = [
            'filerio.in', 'drop.download', 'download.gg', 'uppit.com',
            'uploadbox.io'
        ]
        limit_n = len(Shared.all_links)
        file_size = Path(self.args.input).stat().st_size / 1e+6
        for x in selected_hosts_lst:
            if self.args.number and int(self.args.number) <= limit_n:
                selected_hosts_lst.remove(x)
                continue
            if file_size > more_links['multiup'][x]['limit']:
                selected_hosts_lst.remove(x)
            else:
                limit_n += 1
        if not selected_hosts_lst:
            return

        if self.args.debug:
            s = ''
        else:
            s = 's'
        selected_hosts = ' '.join(
            [f'-F {x}=true' for x in selected_hosts_lst])
        upload = cURL_request(
            f'curl -{s}F "files[]=@{self.args.input}" {selected_hosts}'
            f' {server}')
        if len(upload['files']) == 0:
            upload = cURL_request(f'curl -{s}F "files[]=@{self.args.input}" '
                                  f'{server}')
        link = upload['files'][0]['url'].replace(
            'download', 'en/mirror')
        driver.get(link)

        i = 0
        j = 0
        es_len = None
        elements = []

        while True:
            time.sleep(1)
            if es_len != len(elements):
                j = 0
            driver.refresh()
            elements = driver.find_elements_by_class_name('host')
            es_len = len(elements)
            if es_len == 2:
                i += 1
            else:
                j += 1
            if (j > i + 10) or (len(elements) == len(selected_hosts_lst) + 1):
                break

        for e in elements:
            if '(0)' in e.text:
                link = e.get_attribute('link')
                Shared.all_links.append(link)
                multiup_links.append(link)
                console.print(f'[[{Dp.g}] OK [/{Dp.g}]]', link)

        driver.quit()

        return multiup_links

    def upload(self, headless=True):
        if self.args.debug:
            driver = StartDrive(
                cur_module=Path(__file__).name).start_driver(headless)
        else:
            driver = StartDrive(None).start_driver()
        mirroredto_urls = self._multiup(driver)
        return mirroredto_urls



#!/usr/bin/env python3
# coding: utf-8

import os
import platform
import shlex
import subprocess
import time
from pathlib import Path
from typing import Any

from dracula import DraculaPalette as Dp
from selenium import webdriver
from selenium.common.exceptions import WebDriverException
from selenium.webdriver.firefox.options import Options
from webdriver_manager.firefox import GeckoDriverManager

from .config import Config
from .helpers import Shared, console, selenium_exceptions


class StartDrive:
    def __init__(self, cur_module: Any = None) -> None:
        self.cur_module = cur_module

    @staticmethod
    def download_ublock() -> None:
        Path(Path(Config.ublock).parent).mkdir(exist_ok=True)
        latest = 'https://addons.mozilla.org/firefox/downloads/file/3806442'
        p = subprocess.Popen(
            shlex.split(f'curl -sLo "{Config.ublock}" {latest}'),
            stdout=subprocess.PIPE,
            shell=False
        )
        p.communicate()
        console.print(
            '\nYou\'re running the "--more-links" flag '
            'for the first time. Please wait until everything is ready. '
            'This is a one-time thing.\n',
            style='#f1fa8c')
        if not Path(Config.ublock).exists():
            raise AssertionError

    def start_driver(self, headless: bool = True):
        if not Path(Config.ublock).exists():
            self.download_ublock()

        options = Options()
        profile = webdriver.FirefoxProfile()
        profile.set_preference('media.volume_scale', '0.0')
        if headless:
            options.headless = True
        try:
            driver = webdriver.Firefox(options=options,
                                       firefox_profile=profile,
                                       service_log_path=os.path.devnull)
        except WebDriverException:
            os.environ['WDM_LOG_LEVEL'] = '0'
            os.environ['WDM_PRINT_FIRST_LINE'] = 'False'
            os.environ['WDM_LOCAL'] = '1'
            try:
                driver = webdriver.Firefox(
                    executable_path=GeckoDriverManager().install(),
                    options=options,
                    firefox_profile=profile,
                    service_log_path=os.path.devnull)
            except ValueError:
                raise Exception('Could not find Firefox!')
            except OSError:
                if platform.node() == 'raspberrypi':
                    raise Exception(
                        'It seems like you\'re on a raspberrypi. '
                        'You will need to build gecko driver for ARM: '
                        'https://firefox-source-docs.mozilla.org/testing/'
                        'geckodriver/ARM.html')
        except selenium_exceptions as se:
            console.print(f'[{Dp.b}]{__file__}[{Dp.b}] '
                          f'[{Dp.y}]raised an unexpected issue! '
                          'Try again or remove the `--more-links` flag')
            raise se

        ublock_exists = False
        driver.install_addon(Config.ublock, temporary=True)  # noqa
        time.sleep(1)
        driver.get('about:support')
        body = driver.find_element_by_id('addons-tbody')
        rows = body.find_elements_by_tag_name('tr')
        for row in rows:
            cells = row.find_elements_by_tag_name('td')
            for cell in cells:
                if 'uBlock' in cell.text:
                    ublock_exists = True
        if not ublock_exists:
            raise AssertionError

        capabilities = driver.capabilities
        pid = capabilities['moz:processID']
        Shared.pids.append(pid)
        if self.cur_module is not None:
            console.print(f'Spawned a driver in {self.cur_module}: {pid}')
        return driver



#!/usr/bin/env python3
# coding: utf-8

import sys
import time
import traceback
from typing import NoReturn

import psutil
from dracula import DraculaPalette as Dp

from .helpers import console, logger, Shared


class FirefoxInterruptHandler:
    def __init__(self, pids: list) -> None:
        self.pids = pids

    def firefoxInterrupt(self) -> list:
        terminated = []
        if self.pids:
            for pid in self.pids:
                try:
                    p = psutil.Process(pid)
                    p.terminate()
                    terminated.append(pid)
                except psutil.NoSuchProcess:
                    continue
        return terminated


class KeyboardInterruptHandler:
    def __init__(self) -> None:
        pass  # noqa

    def keyboardInterruptHandler(self) -> NoReturn:
        sys.tracebacklimit = 0
        print('', end='\r')
        time.sleep(0.5)
        console.print(f'[{Dp.y}]Quitting...')
        terminated = FirefoxInterruptHandler(Shared.pids).firefoxInterrupt()
        logger.info('Interrupted by the user.')
        if terminated:
            logger.info('Killed interrupted driver instances:', terminated)
        sys.exit(0)


def SeleniumExceptionInfo(exception: Exception) -> str:
    tb = traceback.format_exception(None, exception,
                                    exception.__traceback__)
    tb = ''.join(tb).rstrip()
    logger.error(f'Selenium encountered an error:\n{tb}')
    return tb


def custom_error_traceback(exception: Exception,
                           error_msg: str,
                           log: bool = False) -> list:
    if log:
        logger.error(error_msg)
    tb = traceback.format_exception(None, exception,
                                    exception.__traceback__)
    if log:
        logger.error(f'{"-" * 10} Start of error traceback {"-" * 10}')
        for ln in tb:
            logger.error(ln.replace('\n', ''))
        logger.error(f'{"-" * 10} End of error traceback {"-" * 10}')
    return tb



#!/usr/bin/env python3
# coding: utf-8

import argparse

from .__init__ import __version__


class CustomHelpFormatter(argparse.HelpFormatter):
    def __init__(self, prog):
        super().__init__(prog, max_help_position=40, width=80)

    def _format_action_invocation(self, action):
        if not action.option_strings or action.nargs == 0:
            return super()._format_action_invocation(action)  # noqa
        default = self._get_default_metavar_for_optional(action)
        args_string = self._format_args(action, default)
        return ', '.join(action.option_strings) + ' ' + args_string


def fmt(prog):
    return CustomHelpFormatter(prog)


def cli() -> argparse.Namespace:
    # noinspection PyTypeChecker
    parser = argparse.ArgumentParser(prog='pymirror', formatter_class=fmt,
                                     add_help=False)
    parser.add_argument('-h',
                        '--help',
                        action='help',
                        default=argparse.SUPPRESS,
                        help='Show this help message and exit')
    parser.add_argument('-i',
                        '--input',
                        help='Path to the input file/folder',
                        required=True)
    parser.add_argument('-s',
                        '--style',
                        help='Output style (default: lines)',
                        choices=['lines', 'list', 'markdown', 'reddit'],
                        default='lines')
    parser.add_argument(
        '-m',
        '--more-links',
        help='Use mirrored.to to generate more likes (default: False)',
        action='store_true',
        default=False)
    parser.add_argument(
        '-n',
        '--number',
        help='Select a specific number of servers to use (default: max)',
        default=None)
    parser.add_argument(
        '-d',
        '--delete',
        help='Delete the file after the process is complete (default: False)',
        action='store_true',
        default=False)
    parser.add_argument(
        '-c',
        '--check-status',
        help='Check the status of the remote servers (default: False)',
        action='store_false',
        default=False)
    parser.add_argument(
        '-D',
        '--debug',
        help='Debug',
        action='store_true',
        default=False)
    parser.add_argument(
        '-l',
        '--log',
        help='Show logs and save it to a file (default: False)',
        action='store_true',
        default=False)
    parser.add_argument(
        '-e',
        '--experimental',
        help='Generate even more links (experimental) (default: False)',
        action='store_true',
        default=False)
    parser.add_argument('-v',
                        '--version',
                        action='version',
                        version=f'%(prog)s {__version__}')

    args = parser.parse_args()
    return args



#!/usr/bin/env python3
# coding: utf-8

import argparse
import inspect
import itertools
import signal
import statistics
import time
from pathlib import Path
from typing import Optional

from dracula import DraculaPalette as Dp

from ..helpers import Shared, console, selenium_exceptions
from ..start_driver import StartDrive


class MoreLinks:
    def __init__(self, args: argparse.Namespace) -> None:
        self.args = args
        self.methods = inspect.getmembers(self, predicate=inspect.ismethod)[1:]
        self.file = str(Path(self.args.input).resolve())
        self.file_size = Path(self.args.input).stat().st_size / 1e+6
        self.headless = True

    def getdict_(self):
        return dict(self.methods)

    @staticmethod
    def init_driver():
        return StartDrive(None).start_driver()

    def upload_to_all_(self):
        times = []
        links = []
        sites = {k: v for k, v in MoreLinks(
            self.args).getdict_().items() if not k.endswith('_')}

        if (
                self.args.number
                and int(self.args.number) < len(Shared.all_links) + 5
        ):
            left = int(self.args.number) - len(Shared.all_links)
            sites = dict(itertools.islice(sites.items(), left))

        for name, function in sites.items():
            try:
                signal.signal(signal.SIGALRM, lambda x, y: 1 / 0)
                start = time.time()
                if len(times) > 2:
                    signal.alarm(int(statistics.mean(times)) + 5)
                link = function()
                times.append(time.time() - start)
                Shared.all_links.append(links)
                links.append(link)
                console.print(f'[[{Dp.g}] OK [/{Dp.g}]]', link)
            except selenium_exceptions:
                console.print(
                    f'[[{Dp.r}] ERROR! [/{Dp.r}]]',
                    f'Encountered error while attempting to upload to'
                    f'[{Dp.b}]{name}[/{Dp.b}]'
                )
            finally:
                signal.alarm(0)
        return links

    def usaupload(self) -> Optional[str]:
        if self.file_size > 1000:
            return
        driver = self.init_driver()
        driver.get('https://usaupload.com/register_non_user')
        time.sleep(2)
        driver.find_element_by_id('add_files_btn').send_keys(self.file)
        driver.find_element_by_class_name('upload-button').click()
        while True:
            try:
                link = driver.find_element_by_class_name(
                    'col-xs-4').get_attribute('dtfullurl')
                if link:
                    driver.quit()
                    return link
            except selenium_exceptions:
                time.sleep(1)

    def filesharego(self) -> Optional[str]:
        if self.file_size > 5000:
            return
        driver = self.init_driver()
        driver.get('https://www.filesharego.com')
        time.sleep(2)
        for e in driver.find_elements_by_class_name('nav-item'):
            if 'Upload File' in e.text:
                e.click()
                break
        time.sleep(2)
        driver.find_element_by_class_name('dz-hidden-input').send_keys(
            self.file)
        while True:
            try:
                link_id = driver.find_element_by_id('copy').get_attribute(
                    'data-id')
                link = driver.find_element_by_id(link_id).get_attribute(
                    'value')
                if link:
                    driver.quit()
                    return link
            except selenium_exceptions:
                time.sleep(1)

    def filepizza(self) -> Optional[str]:
        driver = self.init_driver()
        driver.get('https://file.pizza/')
        driver.find_element_by_css_selector(
            '.select-file-label > input:nth-child(1)').send_keys(self.file)
        while True:
            time.sleep(1)
            link = driver.find_element_by_class_name('short-url').text
            if link:
                driver.quit()
                break
        link = link.replace('or, for short: ', '')
        return link

    def expirebox(self) -> Optional[str]:
        if self.file_size > 200:
            return
        driver = self.init_driver()
        driver.get('https://expirebox.com/')
        time.sleep(2)
        driver.find_element_by_id('fileupload').send_keys(self.file)
        while True:
            time.sleep(1)
            try:
                link = driver.find_element_by_css_selector(
                    'div.input-group:nth-child(3) > input:nth-child(1)'
                ).get_attribute('value')
                if link:
                    driver.quit()
                    return link
            except selenium_exceptions:
                time.sleep(1)

    def filepost(self) -> Optional[str]:
        if self.file_size > 3000:
            return
        driver = self.init_driver()
        driver.get('https://filepost.io/')
        driver.find_element_by_css_selector(
            '.drop-region > input:nth-child(4)').send_keys(self.file)
        while True:
            time.sleep(1)
            try:
                e = driver.find_element_by_css_selector(
                    'div.buttons:nth-child(3) > a:nth-child(1)')
                link = e.get_attribute('href').split('&body=')[1]
                if link:
                    driver.quit()
                    return link
            except selenium_exceptions:
                time.sleep(1)

    def sendcm(self) -> str:
        driver = self.init_driver()
        driver.find_element_by_id('file_0').send_keys(self.file)
        up = driver.find_element_by_id('upload_controls')
        up.find_element_by_class_name('btn').click()
        while True:
            try:
                link = driver.find_element_by_css_selector(
                    '.input-group > textarea:nth-child(2)').text
                return link
            except selenium_exceptions:
                time.sleep(1)



#!/usr/bin/env python3
# coding: utf-8

import argparse
import concurrent.futures
import json
import time
from pathlib import Path

from dracula import DraculaPalette as Dp
from loguru import logger
from selenium.webdriver.common.keys import Keys

from .config import Config
from .handlers import SeleniumExceptionInfo
from .helpers import Shared, console, logger, selenium_exceptions
from .start_driver import StartDrive


class Mirroredto:
    def __init__(self, args: argparse.Namespace):
        self.args = args

    def _mirroredto(self, drivers: list) -> list:
        def process(driver, batch):
            file_size = Path(self.args.input).stat().st_size / 1e+6
            with open(f'{Config.data_path}/more_links.json') as j:
                more_links = json.load(j)

            local_limit = len(Shared.all_links)
            mirroredto_links = []
            try:
                if self.args.number:
                    if local_limit >= int(self.args.number):
                        return
                    local_limit += 1
                time.sleep(2)
                driver.get('https://www.mirrored.to/')
                time.sleep(5)
                html = driver.find_element_by_tag_name('html')
                _ = [html.send_keys(Keys.ARROW_DOWN) for _ in range(3)]

                for x in batch:
                    try:
                        if len(batch) > 8 and x == 'GoFileIo':
                            driver.find_element_by_id(x.lower()).click()
                        elif more_links['mirroredto'][x]['limit'] \
                                > file_size:
                            driver.find_element_by_id(x.lower()).click()
                    except selenium_exceptions as e:
                        console.print(SeleniumExceptionInfo(e))
                        continue

                time.sleep(2)
                resolved_file = str(Path(self.args.input).resolve())
                driver.find_element_by_css_selector(
                    '#uploadifive-html_file_upload > '
                    'input[type=file]:nth-child(3)'
                ).send_keys(resolved_file)
                time.sleep(1)

                driver.find_element_by_id('upload_button').click()
                time.sleep(5)

                while True:
                    if driver.current_url != 'https://www.mirrored.to/':
                        time.sleep(1)
                        break

                link = driver.find_element_by_class_name('mlink').text
                driver.get(link)
                time.sleep(2)
                driver.find_element_by_class_name('secondary').click()
                time.sleep(2)

                start = time.time()

                while True:
                    time.sleep(5)
                    status = [
                        x.text for x in driver.find_elements_by_class_name(
                            'id_Success')
                    ]
                    if len(status) >= 8 or time.time() - start > 60:
                        break

                for x in driver.find_elements_by_class_name('get_btn'):
                    try:
                        x.click()
                    except selenium_exceptions:
                        continue

                current_window = driver.current_window_handle
                for handle in driver.window_handles:
                    driver.switch_to.window(handle)
                    try:
                        LINK = driver.find_element_by_class_name(
                            'code_wrap').text
                        Shared.all_links.append(LINK)
                        mirroredto_links.append(LINK)
                        console.print(f'[[{Dp.g}] OK [/{Dp.g}]] {LINK}')
                        logger.info(f'[ OK ] {LINK}')
                        if handle != current_window:
                            driver.close()
                    except selenium_exceptions:
                        pass

            except Exception as exception:
                raise exception

            finally:
                driver.quit()

            return mirroredto_links

        first_batch = [
            'GoFileIo', 'TusFiles', 'OneFichier', 'ZippyShare', 'UsersDrive',
            'BayFiles', 'AnonFiles', 'ClicknUpload'
        ]
        second_batch = [
            'GoFileIo', 'DownloadGG', 'TurboBit', 'Uptobox', 'SolidFiles',
            'DailyUploads', 'UploadEe', 'DropApk', 'MixdropCo', 'FilesIm',
            'MegaupNet', 'dlupload', 'file-upload'
        ]

        if self.args.debug:
            batches = [first_batch]
        else:
            batches = [first_batch, second_batch]

        with concurrent.futures.ThreadPoolExecutor() as executor:
            mirroredto_urls = []
            results = [
                executor.submit(process, driver, batch)
                for driver, batch in zip(drivers, batches)
            ]
            for future in concurrent.futures.as_completed(results):
                mirroredto_urls.append(future.result())

        return mirroredto_urls

    def upload(self, headless=True):
        if self.args.debug:
            driver = StartDrive(cur_module=Path(__file__).name).start_driver
            drivers = [driver(headless)]
        else:
            driver = StartDrive(None).start_driver
            drivers = [driver(headless), driver(headless)]
        mirroredto_urls = self._mirroredto(drivers)
        return mirroredto_urls



#!/usr/bin/env python3
# coding: utf-8

import argparse
import json
import mimetypes
import platform
import shlex
import signal
import statistics
import subprocess
import time
from pathlib import Path
from typing import Optional, Generator

from dracula import DraculaPalette as Dp

from .handlers import custom_error_traceback
from .helpers import Shared, logger, console


class Error(Exception):
    pass


class APIUpload:
    def __init__(self, data: dict, args: argparse.Namespace) -> None:
        self.data = data
        self.args = args

    @staticmethod
    def ping(ip: str) -> bool:
        def _ping(flag: str, _ip: str) -> int:
            out = subprocess.Popen(shlex.split(f'ping {flag} 1 {_ip}'),
                                   stdout=subprocess.PIPE, shell=False)
            out.communicate()
            return out.returncode

        if platform.system() == 'Windows':
            response = _ping('-n', ip[0])
        else:
            response = _ping('-c', ip[0])
        if response in [0, 256, 512]:
            console.print(
                f'[[{Dp.g}] OK [/{Dp.g}]] [{Dp.c}]{ip[1]}[/{Dp.c}] is online!')
            logger.info(f'{ip[1]} is online!')
            return True
        console.print(
            f'[[{Dp.r}] ERROR! [/{Dp.r}]] [{Dp.c}]{ip[1]}[/{Dp.c}] is '
            'down!'
        )
        logger.warning(f'{ip[1]} is offline!')
        return False

    def curl(self, server: str) -> Optional[str]:
        def find_value(key: str, d: dict) -> Generator[str, None, None]:
            for k, v in (
                    d.items() if type(d) is dict else
                    enumerate(d) if type(d) is list else []
            ):
                if k == key:
                    yield v
                if type(v) in [dict, list]:
                    for value in find_value(key, v):
                        yield value

        if server in ['midi']:
            mimetypes.init()
            mime = mimetypes.guess_type(self.args.input)[0]
            if mime is not None:
                mime = mime.split('/')[0]
                if mime in ['audio', 'video', 'image']:
                    pass
                else:
                    return

        file_size = Path(self.args.input).stat().st_size / 1e+6
        size_limit = self.data[server]['limit']
        if file_size > size_limit:
            return
        srv = self.data[server]['server']
        keys = self.data[server]['keys']
        flags = self.data[server]['flags']
        parameter = self.data[server]['parameter']
        cURL = f'curl {flags} "{parameter}{self.args.input}" {srv}'
        cURL = shlex.split(cURL)
        out = subprocess.run(cURL, stdout=subprocess.PIPE)
        try:
            link = json.loads(out.stdout)
        except json.decoder.JSONDecodeError:
            link = out.stdout.decode('UTF-8').strip('\n')
        if keys:
            link = list(find_value(keys[-1], link))
            if link:
                link = link[0]
        if server == 'oshi':
            link = link.split('\n')[1].replace('DL: ', '')
        return link

    def api_uploads(self, responses: list) -> Optional[list]:
        times = []
        api_uploads_links = []
        for n, ((k, _), res) in enumerate(zip(self.data.items(), responses)):
            if self.args.number and n == int(self.args.number):
                break
            if self.args.debug and n == 1:
                break
            if res is False:
                continue
            try:
                signal.signal(signal.SIGALRM, lambda x, y: 1 / 0)
                start = time.time()
                if len(times) > 2:
                    signal.alarm(int(statistics.mean(times)) + 5)
                link = self.curl(k)
                if not link:
                    continue
                if (
                    'bad gateway' in link.lower()
                    or 'error' in link.lower()
                    or 'https://' not in link.lower()
                    ):
                    continue
                console.print(f'[[{Dp.g}] OK [/{Dp.g}]]', link)
                Shared.all_links.append(link)
                api_uploads_links.append(link)
                logger.info(f'[ OK ] {link}')
                times.append(time.time() - start)
            except ZeroDivisionError:
                if self.args.log:
                    logger.error(f'{k} Timed out!')
            except Error as e:
                custom_error_traceback(e,
                                       f'[ ERROR! ] Error in {k}...',
                                       log=self.args.log)
            finally:
                signal.alarm(0)
        return api_uploads_links



#!/usr/bin/env python3
# coding: utf-8

import argparse
import json
import multiprocessing
import os
import shutil
import signal
import socket
import sys
import tarfile
import time
from pathlib import Path
from typing import Union, NoReturn

from dracula import DraculaPalette as Dp
from rich.panel import Panel

from .api_upload import APIUpload
from .config import Config
from .experimental.more_links import MoreLinks
from .handlers import KeyboardInterruptHandler
from .helpers import Shared, console, logger, load_data
from .mirroredto import Mirroredto
from .multiup import MultiUp


class PyMirror:
    def __init__(self, args: argparse.Namespace) -> None:
        self.args = args
        self.data = load_data()

    @staticmethod
    def initializer() -> NoReturn:
        signal.signal(signal.SIGINT, signal.SIG_IGN)

    def tgz(self) -> str:
        out_file = f'{Path(self.args.input).parent}/' \
                   f'{Path(self.args.input).name}.tar.gz '
        with tarfile.open(out_file, 'w:gz') as t:
            t.add(self.args.input, arcname=Path(self.args.input).name)
        return out_file

    def return_ips(self) -> list:
        ips = []
        for k, _ in self.data.items():
            try:
                server = self.data[k]['server'].split('/')[2]
            except IndexError:
                server = self.data[k]['server']
            ip = socket.gethostbyname(server)
            ips.append((ip, k))
        return ips

    def style_output(self, links_dict: dict) -> Union[list, str]:
        names = list(links_dict.keys())
        links = list(links_dict.values())
        style = self.args.style
        if style == 'list':
            output = '\n'.join(links)
        elif style == 'markdown':
            output = '\n'.join(
                [f'- [{name}]({link})' for name, link in zip(names, links)])
        elif style == 'reddit':
            output = ' | '.join(
                [f'[Mirror {n + 1}]({link})' for n, link in enumerate(links)])
        else:
            output = '\n'.join(links)
        return output

    def uploader(self) -> Union[list, str]:
        start_time = time.time()
        signal.signal(signal.SIGINT,
                      KeyboardInterruptHandler.keyboardInterruptHandler)

        if self.args.experimental and not self.args.more_links:
            raise Exception(
                'You need to add the `--more-links` flag to use '
                '`--experimental`')

        with open(f'{Config.data_path}/servers_data.json') as j:
            data = json.load(j)

        if self.args.log:
            logger.remove()
            logger.add(Config.log_file, level='DEBUG')
            logger.add(sys.stderr, level='ERROR')

        console.print('Press `CTRL+C` at any time to quit.', style='#f1fa8c')

        if self.args.check_status:
            console.rule('Checking servers status...')
            ips = self.return_ips()
            cpus = multiprocessing.cpu_count() - 1
            with multiprocessing.Pool(cpus, initializer=self.initializer) as p:
                responses = p.map(APIUpload.ping, ips)
        else:
            responses = []

        console.rule('Uploading...')

        file = self.args.input
        if not Path(file).exists():
            raise FileNotFoundError('If you are sure the path is correct, '
                                    'rename the file removing any illegal '
                                    'characters, then try again.')
        if Path(file).is_dir():
            file = self.tgz()

        if len(responses) == 0:
            responses = [True] * len(data.keys())

        APIUpload(self.data, self.args).api_uploads(responses)

        if self.args.more_links:
            Mirroredto(self.args).upload()
            MultiUp(self.args).upload()
            if self.args.experimental:
                MoreLinks(self.args).upload_to_all_()

        links_dict = {}
        for link in Shared.all_links:
            try:
                domain = link.split('/')[:-1][2].replace('www.', '')
                name = '.'.join(domain.split('.')[0:])
                if len(name) == 1:
                    name = '.'.join(domain.split('.')[1:])
                elif len(name.split('.')) == 3:
                    name = '.'.join(domain.split('.')[1:])
                links_dict.update({name: link})
            except IndexError:
                continue

        output = self.style_output(links_dict)

        if self.args.delete:
            if Path(self.args.input).is_dir():
                shutil.rmtree(self.args.input)
            else:
                os.remove(file)
        if Path(file).suffixes == ['.tar', '.gz']:
            os.remove(file)
        console.rule(f'Results: {len(Shared.all_links)}')
        print(output)
        os.rename(file, file)
        for x in Shared.all_links:
            logger.info(x)
        run_time = time.strftime('%H:%M:%S',
                                 time.gmtime(time.time() - start_time))
        h, m, s = [int(_) for _ in run_time.split(':')]
        console.print(
            Panel.fit(f'[{Dp.k}]Process took[{Dp.k}] [{Dp.y}]{h}h {m}m {s}s'))
        console.rule('END')
        return output



#!/usr/bin/env python3
# coding: utf-8

import argparse
import inspect
import json
import re
from pathlib import Path

import loguru
import selenium.common.exceptions
from rich.console import Console

from .config import Config


class Shared:
    all_links = []
    pids = []


def logger():
    logger_ = loguru.logger
    logger_.remove()
    return logger_


def selenium_exceptions_classes():
    selenium_exceptions_list = inspect.getmembers(selenium.common.exceptions,
                                                  predicate=inspect.isclass)
    return list(zip(*selenium_exceptions_list))[1]


def _match_links(links_raw: list) -> list:
    regex = re.compile(
        r'^(?:http|ftp)s?://'
        r'(?:(?:[A-Z0-9](?:[A-Z0-9-]{0,61}[A-Z0-9])?\.)+'
        r'(?:[A-Z]{2,6}\.?|[A-Z0-9-]{2,}\.?)|'
        r'localhost|'
        r'\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})'
        r'(?::\d+)?'
        r'(?:/?|[/?]\S+)$', re.IGNORECASE)

    links = []
    for link in links_raw:
        if re.match(regex, link) is not None:
            links.append(link)
    return links


def load_data():
    with open(f"{Config.data_path}/servers_data.json") as j:  # noqa
        data = json.load(j)
    return data


def download_time(file: str) -> float:
    file_size = Path(file).stat().st_size / 1e+6
    return file_size / float(Config['main']['upload_speed'])


console = Console()
logger = logger()
selenium_exceptions = selenium_exceptions_classes()
Namespace = argparse.Namespace
